from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from xml.sax import make_parser, handler, SAXException
from xml.sax.saxutils import *
from xbelplone import ImportHandler
import random, string
from DateTime import DateTime

""" 
   ATBookmarks allows you to manage your Bookmarks online in a Folder
"""

schema = BaseSchema +  Schema((
        StringField('id',
                 required=1, 
                 mode="rw",
                 accessor="getId",
                 mutator="setId",
                 default=None,
                 widget=IdWidget(
                 label="Short Name",
                 label_msgid="label_short_name",
                 description="Should not contain spaces, underscores or mixed case. "\
                 "Short Name is part of the item's web address.",
                 description_msgid="help_shortname",
                 visible={'view' : 'visible'},
                 i18n_domain="plone"),
                 ),
    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label="Title",
                                    label_msgid="label_title",
                                    description_msgid="help_title",
                                    i18n_domain="plone"),
                ),    
    StringField('description',
                isMetadata=1,
                accessor='Description',
                widget=TextAreaWidget(label='Description', 
                                label_msgid="label_Description",
                                description='A brief description of this item.',
                                description_msgid="help_description",
                                i18n_domain="plone"), 
               ),
    ))
    
new_actions = (
    {'id': 'bookmarklivefeed',
     'name': 'bookmark RSS Feed',
     'action': "string:${object_url}/bookmark-rss.xml",
     'permissions': (CMFCorePermissions.View,),
     'category': 'document_actions',
     },
     {'id': 'xbelxml',
     'name': 'xbel XML',
     'action': "string:${object_url}/xbel.xml",
     'permissions': (CMFCorePermissions.View,),
     'category': 'document_actions',
     },
     )
     
def modify_fti(fti):
    fti['actions'] = fti['actions'] + new_actions

        
class BookmarkFolder(OrderedBaseFolder):
    """
        A Bookmark folder lets you manage your Bookmarks online
    """
    schema = schema
    filter_content_types = 1
    allowed_content_types = ('Link', 'Favorite', 'Image', 'BookmarkFolder') 
    meta_type = 'BookmarkFolder'
    archetype_name = 'Bookmark Folder'
    
    content_icon = 'bookmark_folder.png'
    actions = ({
       'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}/bookmark_view',
        'permissions': (CMFCorePermissions.View,)
        },
       {'id'            : 'upload',
        'name'          : 'Upload',
        'action'        : 'string:${object_url}/upload_xbel_form',
        'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
        'visible':0
         },
        {'id': 'references',
          'name': 'References',
          'action': 'string:${object_url}/reference_edit',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'visible':0},
        {'id': 'metadata',
          'name': 'Properties',
          'action': 'string:${object_url}/base_metadata',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'visible':0})    
          
    
        
    def fromFile(self, stream):
        """ Import bookmarks from a file into this folder """
        m = ImportHandler()
        parser = make_parser()
        parser.setContentHandler(m)
        parser.parse(stream)
        f = m.currentFolder
        self.convertFolder(self, f)
        
    def convertFolder(self, ploneFolder, xbelFolder):
        """ Add links and BookmarkFolders to current folder """
        xbelFolder.bookmarks.sort()
        #xbelFolder.bookmarks.reverse()
        description = ""
        for b in xbelFolder.bookmarks:
                try:
                    id = b.id
                except:
                    id =  'lnk_' + str(random.randint(1000,99999))
                title=b.name
                url=b.url
                description=b.desc
                try: 
                    ploneFolder.invokeFactory('Link', id=id, title=title, remote_url=url, description=description)
                except:
                    pass
                
        for f in xbelFolder.subfolders:
            try:
                ploneFolder.invokeFactory('BookmarkFolder', id=f.id, title=f.name, description=f.desc)            
            except:
                pass
            try:
                folder = getattr(ploneFolder, f.id)
                self.convertFolder(folder, f)
            except:
                pass
         
registerType(BookmarkFolder)
